#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <string>

#include "sld.hh"

string binpath;

int main(int argc, char *argv[])
{
  SLDaemon &daemon = SLDaemon::Instance();
  SLDaemon::Options *lineopt;
  SLDaemon::Options *fileopt = NULL;

  try {
    //parse arg line options
    lineopt = daemon.getoptions(argc, argv);

    assert(lineopt);
    if (lineopt->conffile != "") {
      //parse conf file options
      fileopt = daemon.getoptions(lineopt->conffile);
    }
  }
  catch (const Error &e) {
    e.print();
    return e.code();
  }
  try {
    //apply config file option
    if (fileopt)
      daemon.applyoptions(fileopt);

    //apply parsed option
    daemon.applyoptions(lineopt);

    // register sigchild handler
    if (signal(SIGCHLD, sigchild) == SIG_ERR) {
      perror("sld: error");
      return ERR_UNKNOWN;
    }

    //execute receive command
    daemon.run();
  }
  catch (const Error &e) {
    std::cerr <<"sld: error: " << e.message() << "." << std::endl;
    return e.code();
  }

  return ERR_UNKNOWN;
}

string getbinpath() {
  int pid = getpid();
  char buf[PATH_MAX];
  char buf2[PATH_MAX] = "";

  snprintf(buf, PATH_MAX, "/proc/%d/exe", pid);
  if (readlink(buf, buf2, PATH_MAX) == 0) {
    perror("sld: getbinpath: readlink: ");
    return (string) "";
  }
  return (string) buf2;
}

bool cp(const char *src, const char *dst) {
  assert(src);
  assert(dst);

  //open source file
  FILE *fsrc = fopen(src, "r");
  if (!fsrc)
    return false;

  //open destination file
  FILE *fdst = fopen(dst, "w");
  if (!fdst) {
    fclose(fsrc);
    return false;
  }

  //do copy
  char buf[512];
  size_t nread;
  size_t nwrite;
  while ((nread = fread(buf, 1, 512, fsrc)) > 0) {
    nwrite = fwrite(buf, 1, nread, fdst);
    if (nwrite != nread) {
      fclose(fsrc);
      fclose(fdst);
      return false;
    }
  }
  fclose(fsrc);
  fclose(fdst);
  return true;
}
