#ifndef DAEMON_HH
# define DAEMON_HH

typedef std::string string;
typedef std::stringstream sstream;
typedef std::ostream ostream;
typedef std::ifstream ifstream;

// -----------------------------------------------------------------------------
// SLDAEMON CLASS
// -----------------------------------------------------------------------------

class SLDaemon
{
public:

  static const string version;

  static SLDaemon &Instance();

  struct Options {
    Options();
    void print();
    string server;
    int port;
    string login;
    string pass;
    int verbose; //0=false, 1=true, 3=undef
    string scriptdir;
    string conffile;
  };

  struct Job {
    Job() {}
    Job(int, const string &, time_t);
    int pid;
    string name;
    time_t start_time;
    bool operator()(const Job &a, const Job &b);
  };

  inline bool verbose() const { return options_.verbose == 1; }

  Options *getoptions(int argc, char *argv[]) const;
  Options *getoptions(const string file) const;
  void applyoptions(const Options *opt);
  void run();

  // friends functions

  friend void sigchild(int);

protected:
  // parsed options
  Options options_;

  // current socket
  int socket_fd_;
  FILE* socket_fs_;

  // pid list
  typedef std::set<Job, Job> t_job;
  t_job jobs_;

  // Options functions
  void usage(const char *argv0) const;
  void check_options() const;

  // network functions
  void connect();
  void disconnect();
  void send(const char *data, size_t len, bool buf = true);
  void send(const string str, bool buf = true);
  void send(long int i);
  void sendln(const string &s);
  char *recv(size_t size);
  void recv(size_t size, const string &filename);
  char *recvln();
  void flush();

  // protocol functions
  void auth();

  // commmand functions
  void cmd_exit();
  void cmd_version();
  void cmd_list();
  void cmd_status();
  void cmd_killall();
  void cmd_kill(const char *line);
  void cmd_exec(const char *line);
  void cmd_update();
  void cmd_file(const char *line);

  // others functions
  string md5(const string &file) const;
  void clean_dir(const string &dir) const;

private:
  static SLDaemon *instance_;

  SLDaemon();
  ~SLDaemon() {}
  SLDaemon(const SLDaemon &) { assert(true); }
  SLDaemon &operator=(const SLDaemon &) { assert(true); return *instance_; }
};

#endif
