//
// getopt.cc for sls
//
// Made by Seblu
// Login   <seblu@epita.fr>
//
// Started on  Tue Jun  5 22:36:50 2007 Seblu
// Last update Thu Jun  7 02:10:37 2007 Seblu
//

#include <stdlib.h>
#include <errno.h>
#include "sls.hh"

const int GetOption::min_argc = 4;

void GetOption::usage(const char *argv0)
{
  //print usage
  std::cerr << "Usage: "
	    << *argv0
	    << " -f <acces file> -p <port number> [-v] [-h]"
	    << std::endl;
  std::cerr << "-f : password map file" << std::endl;
  std::cerr << "-p : Port where server listen" << std::endl;
  std::cerr << "-v : Verbose mode" << std::endl;
  std::cerr << "-h : Display this messages" << std::endl;
}

GetOption::GetOption(int argc, char *argv[]) : port_ (0), verbose_ (false)
{
  // parse arg line
  for (int i = 1; i < argc; ++i) {
    if (!strcmp(argv[i], "-h")) {
      usage(*argv);
      exit(EXIT_OK);
    }
    else if (!strcmp(argv[i], "-v")) {
      verbose_ = true;
    }
    else if (!strcmp(argv[i], "-p")) {
      if (++i >= argc)
	throw Error("No enough argument for option -p.");
      char *endptr;
      port_ = strtol(argv[i], &endptr, 10);
      if (!(*argv[i] != '\0' && *endptr == '\0'))
	throw Error("Unable to convert port to a number.");
    }
    else if (!strcmp(argv[i], "-f")) {
      if (++i >= argc)
	throw Error("No enough argument for option -f.");
      accessfile_ = string(argv[i]);
    }
    else {
      Error *err = new Error();
      *err << "Invalid options : " << argv[i] << ".";
      throw *err;
    }
  }

  // Check validy of arguement
  if (port_ == 0)
    throw Error("No server port specified.");
  if (port_ < 1 || port_ > 65535)
    throw Error("Bad server port number (1 - 65535).");
  if (accessfile_ == "")
    throw Error("No access file specified.");

  // print info in verbose mode
  if (verbose_) {
    std::cout << "Server listen port is : " << port_ << "." << std::endl;
    std::cout << "Server access file : " << accessfile_ << "." << std::endl;
    std::cout << "Verbose mode : " << verbose_ << "." << std::endl;
  }
}

int
GetOption::port() const
{
  return port_;
}

const string&
GetOption::accessfile() const
{
  return accessfile_;
}

bool
GetOption::verbose() const
{
  return verbose_;
}
